@@ -1,10 +1,12 @@
 from twisted.trial.unittest import TestCase
 from twisted.python.modules import getModule
+from twisted.web.resource import Resource, getChildForRequest
 
 from shutil import copy
 
 from saratoga.api import SaratogaAPI
 from saratoga.outputFormats import OutputRegistry
+from saratoga.test.requestMock import requestMock, _render
 
 import json
 
@@ -22,6 +24,9 @@ class APIImpl(object):
         def exampleregex_GET(self, request, params, someID):
             return {"id": someID}
 
+        def nothing_GET(self, request, params):
+            pass
+
         def __init__(self):
             self.jsonbodyParams_GET = self.example_GET
             self.urlParams_GET = self.example_GET
@@ -41,6 +46,10 @@ APIDef = {
             "endpoint": "example",
             "getProcessors": [{"versions": [1]}]
         },
+        {
+            "endpoint": "nothing",
+            "getProcessors": [{"versions": [1]}]
+        },
         {
             "func": "exampleregex",
             "endpoint": r"example/(\d+)",
@@ -198,21 +207,56 @@ class SaratogaAPITests(TestCase):
     def setUp(self):
         fp = getModule(__name__).filePath
         copy(fp.parent().child("jsonschemaext.json").path, "jsonschemaext.json")
-        
+
         self.api = SaratogaAPI(APIImpl, APIDef)
 
+    def test_EmbeddedSaratogaBasic(self):
+
+        def rendered(res, request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "success", "data": {}}
+            )
+
+        r = Resource()
+        r.putChild("api", self.api.getResource())
+        req = requestMock("/api/v1/nothing")
+        endup = getChildForRequest(r, req)
+        d = _render(endup, req)
+        d.addCallback(rendered, req)
+        return r
+
+    def test_EmbeddedSaratogaWithRegex(self):
+
+        def rendered(res, request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "success", "data": {"id": "4"}}
+            )
+
+        r = Resource()
+        r.putChild("api", self.api.getResource())
+        req = requestMock("/api/v1/example/4")
+        endup = getChildForRequest(r, req)
+        d = _render(endup, req)
+        d.addCallback(rendered, req)
+        return r
+
+
     def test_customOutputFormatRegistry(self):
 
         o = OutputRegistry("application/json")
         api = SaratogaAPI(APIImpl, APIDef, outputRegistry=o)
         self.assertIs(o, api.outputRegistry)
-        
+
+
     def test_getResource(self):
         """
         Check that Saratoga returns the correct resource.
         """
         self.assertIs(self.api.resource, self.api.getResource())
 
+
     def test_basic(self):
         """
         Basic Saratoga test.
@@ -225,6 +269,20 @@ class SaratogaAPITests(TestCase):
 
         return self.api.test("/v1/example").addCallback(rendered)
 
+
+    def test_basicNothing(self):
+        """
+        Double check we handle functons that return nothing.
+        """
+        def rendered(request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "success", "data": {}}
+            )
+
+        return self.api.test("/v1/nothing").addCallback(rendered)
+
+
     def test_basicWithEmptyParams(self):
         """
         Basic Saratoga test.
@@ -238,6 +296,7 @@ class SaratogaAPITests(TestCase):
         return self.api.test("/v1/example", replaceEmptyWithEmptyDict=True
             ).addCallback(rendered)
 
+
     def test_basicRegex(self):
         """
         Basic Saratoga test, testing the regex stuff.
@@ -249,8 +308,8 @@ class SaratogaAPITests(TestCase):
             )
 
         return self.api.test("/v1/example/4").addCallback(rendered)
-        
-        
+
+
     def test_handlingOfExceptions(self):
         """
         Test that throwing a generic exception is handled gracefully.
@@ -437,7 +496,7 @@ class SaratogaAPITests(TestCase):
             warnings = self.flushLoggedErrors()
             self.assertEqual(warnings[0].getErrorMessage(), "u'cake' is a "
                 "required property, u'muffin' is a required property")
-            
+
         return self.api.test("/v1/responseParamsExtLoad").addCallback(rendered)
 
 
@@ -510,3 +569,72 @@ class SaratogaAPITests(TestCase):
             "hello": "yes", "goodbye": "no", "the": "beatles"
             })
         return d.addCallback(rendered)
+
+    def test_undefinedHTTPMethod(self):
+        """
+        Test that an undefined HTTP method returns a nice value.
+        """
+        def rendered(request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "fail", "data": "Method not allowed."}
+            )
+
+        return self.api.test("/v1/example", method="WAFFLE").addCallback(rendered)
+
+
+    def test_customHTTPMethods(self):
+
+        APIDef = {
+            "metadata": {"versions": [1]},
+            "endpoints": [
+                {
+                    "endpoint": "example",
+                    "waffleProcessors": [{"versions": [1]}]
+                }
+            ]
+        }
+
+        class APIImpl(object):
+            class v1(object):
+                def example_WAFFLE(self, request, params):
+                    return "OK"
+
+        self.api = SaratogaAPI(APIImpl, APIDef, methods=["WAFFLE"])
+
+        def rendered(request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "success", "data": "OK"}
+            )
+
+        return self.api.test("/v1/example", method="WAFFLE").addCallback(rendered)
+
+    def test_customHTTPMethodsFunnyCasing(self):
+        """
+        Test funny casing works.
+        """
+        APIDef = {
+            "metadata": {"versions": [1]},
+            "endpoints": [
+                {
+                    "endpoint": "example",
+                    "waffleProcessors": [{"versions": [1]}]
+                }
+            ]
+        }
+
+        class APIImpl(object):
+            class v1(object):
+                def example_WAFFLE(self, request, params):
+                    return "OK"
+
+        self.api = SaratogaAPI(APIImpl, APIDef, methods=["WaFfLE"])
+
+        def rendered(request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "success", "data": "OK"}
+            )
+
+        return self.api.test("/v1/example", method="WAFFlE").addCallback(rendered)
